/*
MIT License

Copyright (c) 2009 Justin (tuxinator2009) Davis

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

#ifndef DUNGEONGENERATOR_H
#define DUNGEONGENERATOR_H

#include <SDL.h>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <string>
#include <map>
#include <vector>
#include "randomnumbergenerator.h"

#define UPDATE_TIME 500
#define MIN_TILE_SIZE 1
#define WALL_ATTEMPTS 1
#define MAX_PLACEMENT_ATTEMPTS 10
#define HALLS_TO_PLACE 1

struct Point
{
	Point()
	{
		x = 0;
		y = 0;
		count = 0;
	}
	int x;
	int y;
	int count;
	Point& operator=(const Point &other);
	bool operator==(const Point &other) const;
	bool operator<(const Point &other) const;
	bool operator>(const Point &other) const;
};

class DungeonGenerator
{
	public:
		DungeonGenerator();
		~DungeonGenerator();
		void setNumRooms(int value);
		void setMinRoomWidth(int value);
		void setMinRoomHeight(int value);
		void setMaxRoomWidth(int value);
		void setMaxRoomHeight(int value);
		void setMinHallLength(int value);
		void setMaxHallLength(int value);
		void generateDungeon(SDL_Surface *screen, unsigned long seed, bool debug);
		void displayDungeon(SDL_Surface *screen);
		void displayDungeon(SDL_Surface *screen, int screenXOffset, int screenYOffset);
		int getMapWidth();
		int getMapHeight();
	private:
		int getRandomPoint();
		bool isSpaceAvailable(int x, int y, int w, int h);
		bool isAdjacentToFloor(Point point);
		void addArea(int x, int y, int w, int h, int surroundSides, int floorType);
		void createMapData();
		int numNeighbors(Point point, int type);
		void cleanDungeon(SDL_Surface *screen);
		void removeWall(Point point);
		void removeWall(int index);
		void displayPercentage(SDL_Surface *screen);
		void processInput();
		enum MapType {SPACE_UNUSED=0, SPACE_WALL, SPACE_REMOVEDWALL, SPACE_FLOOR, SPACE_HALLFLOOR, SPACE_ROOMFLOOR};
		enum Direction {DIR_NORTH=1, DIR_SOUTH=2, DIR_EAST=4, DIR_WEST=8};
		SDL_Surface *numbers;
		RandomNumberGenerator numberGenerator;
		std::map<Point, int> generatedMap;
		std::map<Point, int>::iterator iter;
		std::vector<Point> wallList;
		int *mapData;
		int numRooms;
		int roomsPlaced;
		int minRoomWidth;
		int minRoomHeight;
		int maxRoomWidth;
		int maxRoomHeight;
		int minHallLength;
		int maxHallLength;
		int mapWidth;
		int mapHeight;
		int numRemovedWalls;
		bool debugMode;
};

#endif // DUNGEONGENERATOR_H
